#!/usr/bin/env node

// Required parameters:
// @raycast.schemaVersion 1
// @raycast.title Get Clients
// @raycast.mode fullOutput

// Optional parameters:
// @raycast.icon 🤖
// @raycast.packageName CTS
// @raycast.argument1 { "type": "text", "placeholder": "Username"}
// @raycast.argument2 { "type": "text", "placeholder": "Password", "secure": true}

// Documentation:
// @raycast.description Get Clients
// @raycast.author Josh Willing
// @raycast.authorURL https://github.com/jwillinghalpern

// YOU CAN REMOVE THE STUFF ABOVE IF YOU'RE NOT USING THE RAYCAST APP

// ————————————————————————————————————————————————————————————————————————————————

const server = 'YOUR_SERVER_DOMAIN_HERE'; // e.g. my.fms.com. NOTE, this could be passed in as an argument isntead (see below for syntax)

const usr = process.argv.slice(2)[0];
const pwd = process.argv.slice(2)[1];
const auth = Buffer.from(`${usr}:${pwd}`).toString('base64');
const baseUrl = `https://${server}/fmi/admin/api/v2`;
async function login() {
  var myHeaders = new Headers();
  myHeaders.append('Authorization', `Basic ${auth}`);

  var requestOptions = {
    method: 'POST',
    headers: myHeaders,
    redirect: 'follow',
  };

  return fetch(`${baseUrl}/user/auth`, requestOptions)
    .then((response) => response.json())
    .then((result) => result)
    .catch((error) => console.log('login error', error));
}

async function getClients(token) {
  var myHeaders = new Headers();
  myHeaders.append('Authorization', `Bearer ${token}`);

  var requestOptions = {
    method: 'GET',
    headers: myHeaders,
    redirect: 'follow',
  };

  return fetch(`${baseUrl}/clients`, requestOptions)
    .then((response) => response.json())
    .then((result) => result)
    .catch((error) => console.log('get clients error', error));
}

async function logout(token) {
  var requestOptions = {
    method: 'DELETE',
    redirect: 'follow',
  };

  return fetch(`${baseUrl}/user/auth/${token}`, requestOptions)
    .then((response) => response.text())
    .then((result) => result)
    .catch((error) => console.log('logout error', error));
}

function spacer(lines) {
  return Array(lines).fill('').join('\n');
}

function getDigest(res) {
  const digest = res.response.clients
    .map((client) => {
      const { userName, appType, appVersion, guestFiles } = client;
      const files = guestFiles
        .map((file) => {
          const { accountName, filename, groupName } = file;
          return `    · ${filename}: ${accountName} (${groupName})`;
        })
        .join('\n');
      return `${userName} (${appType} ${appVersion}):\n${files}`;
    })
    .join('\n');

  return (
    '--<>--<>--<>--<>--<>--<>--<>--\n|           DIGEST           |\n--<>--<>--<>--<>--<>--<>--<>--\n' +
    digest
  );
}

async function main() {
  let token;
  try {
    const loginRes = await login();
    token = loginRes.response.token;
    const clients = await getClients(token);
    console.log(JSON.stringify(clients, null, 2));
    console.log(spacer(3));
    console.log(getDigest(clients));
  } catch (error) {
    console.log(error);
  } finally {
    await logout(token);
  }
}

main();
